<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class Initial
 *
 * Created via:
 * oitc migrations create -p DesignModule Initial
 *
 * Run migration:
 * oitc migrations migrate -p DesignModule
 *
 */
class Initial extends AbstractMigration {
    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;


    /**
     * Up Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-up-method
     * @return void
     */
    public function up(): void {
        if (!$this->hasTable('designs')) {
            $this->table('designs')
                ->addColumn('id', 'integer', [
                    'autoIncrement' => true,
                    'default'       => null,
                    'limit'         => 11,
                    'null'          => false,
                ])
                ->addPrimaryKey(['id'])
                ->addColumn('pageHeader', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('pageSidebar', 'string', [ //.page-logo, .page-sidebar, .nav-footer, .bg-brand-gradient
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('pageSidebarGradient', 'string', [ //.page-logo, .page-sidebar, .nav-footer, .bg-brand-gradient
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('navTitle', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('navMenu', 'string', [ //.nav-menu li a  .nav-menu li i
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('navMenuHover', 'string', [ //.nav-menu li a:hover  .nav-menu li i:hover
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('navLink', 'string', [ //.nav-tabs .nav-item .nav-link.active:not(:hover)
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('navLinkHover', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('navTab', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('pageContent', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('pageContentWrapper', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('panelHdr', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('panel', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('breadcrumbNonLinks', 'string', [ // breadcrumb
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('breadcrumbLinks', 'string', [ // breadcrumb links a
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('tableBase', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('tableBaseFont', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('tableBorder', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('tableHover', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('tableHoverFont', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('btnDefault', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('btnDefaultFont', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('btnDefaultBorder', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('cardBase', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('cardBaseFont', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('cardHeader', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('cardHeaderFont', 'string', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])->addColumn('logoInHeader', 'integer', [
                    'default' => null,
                    'limit'   => 255,
                    'null'    => false,
                ])
                ->create();

        }

    }

    /**
     * Down Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-down-method
     * @return void
     */
    public function down(): void {
        $this->table('designs')->drop()->save();
    }

}
