<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace DesignModule\Controller;

use Cake\Http\CallbackStream;
use Cake\ORM\TableRegistry;
use DesignModule\Model\Table\DesignsTable;
use itnovum\openITCOCKPIT\Core\System\FileUploadSize;
use Symfony\Component\Config\Definition\Exception\InvalidTypeException;

/**
 * Designs Controller
 *
 *
 * @method \DesignModule\Model\Entity\Design[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class DesignsController extends AppController {

    /**
     * @var array
     */
    private $defaultTheme = [
        'pageHeader'          => '#ffffff', //background-color
        'pageSidebar'         => '#1c2421', //background color
        'pageSidebarGradient' => '#468af0', //background color
        'navTitle'            => '#7c7e7d',//color
        'navMenu'             => '#b7b7b7',//color .nav-menu li a
        'navMenuHover'        => '#ffffff',//color .nav-menu li a:hover
        'navLink'             => '#727b84',//tabs in e.g. dashboard color .nav-link:not([class^="btn-"]):not([class*=" btn-"])
        'navLinkHover'        => '#333', //tabs in e.g. dashboard color .nav-link.active:not(:hover)
        'navTab'              => '#fff', //navTab active background-color
        'pageContent'         => '#444444', //color (breadcrumbs ohne link)
        'pageContentWrapper'  => '#ebecec', //background color
        'panel'               => '#ffffff',//background color
        'panelHdr'            => '#ffffff',// background
        'breadcrumbNonLinks'  => '#444444', //.breadcrumb-item color
        'breadcrumbLinks'     => '#4285F4', //.breadcrumb-item > a color
        'tableBase'           => '#ffffff', //.table background-color
        'tableBaseFont'       => '#212529', //.table color
        'tableBorder'         => '#f1f3f7', //.table-bordered border-color
        'tableHover'          => '#dadce0', //.table-hover border-color
        'tableHoverFont'      => '#212529', //.table-hover color
        'btnDefault'          => '#f5f5f5', //.btn-default background color (also background-image gradient)
        'btnDefaultFont'      => '#444', //.btn-default color
        'btnDefaultBorder'    => '#d4d4d4', //border-color
        'cardBase'            => '#fff', //.card background-color
        'cardBaseFont'        => '#444', //.card color
        'cardHeader'          => '#f7f9fa', //.card-header background-color
        'cardHeaderFont'      => '#444', //.card-header color
        'logoInHeader'        => 1,
        'customcsstext'       => '',

    ];

    public function edit() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }
        /** @var DesignsTable $DesignsTable */
        $DesignsTable = TableRegistry::getTableLocator()->get('DesignModule.Designs');
        $design = $DesignsTable->getDesign();
        $new = false;
        if (empty($design)) {
            $design = $this->defaultTheme;
            $new = true;
        }
        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            $FileUploadSize = new FileUploadSize();

            //Return user information
            $this->set('design', $design);
            $this->set('new', $new);
            $this->set('maxUploadLimit', $FileUploadSize->toArray());
            $this->viewBuilder()->setOption('serialize', ['design', 'maxUploadLimit', 'new']);
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            if ($new) {
                $design = $DesignsTable->newEntity($this->request->getData('Design', []));
                $design['id'] = 1;
            } else {
                $design = $DesignsTable->patchEntity($design, $this->request->getData('Design', []));
            }

            $DesignsTable->save($design);
            if ($design->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $design->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            try {
                $content = require_once(PLUGIN . 'DesignModule' . DS . 'templates' . DS . 'Designs' . DS . 'css' . DS . 'style.php');
                $cssFile = PLUGIN . 'DesignModule' . DS . 'webroot' . DS . 'css' . DS . 'customStyle.css';
                $DesignsTable->writeStyleFile($cssFile, $content);
            } catch (\Exception $e) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', 'Cannot write style file: ' . $e->getMessage());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $this->set('message', __('Design saved successfully'));
            $this->viewBuilder()->setOption('serialize', ['message']);
            return;
        }


        $this->set('design', $design);
        $this->viewBuilder()->setOption('serialize', ['design']);
    }

    /**
     * export style to json file
     */
    public function export() {
        $this->disableAutoRender();

        /** @var DesignsTable $DesignsTable */
        $DesignsTable = TableRegistry::getTableLocator()->get('DesignModule.Designs');

        $design = $DesignsTable->getDesign();
        if (!empty($design)) {
            unset($design['id']);
        } else {
            $design = $this->defaultTheme;
        }

        $this->response = $this->response->withHeader('Content-Type', 'application/json');
        $this->response = $this->response->withHeader('Content-Disposition', 'attachment; filename=openITCOCKPIT-design.json');
        $stream = new CallbackStream(function () use ($design) {
            echo json_encode($design, JSON_PRETTY_PRINT);
        });
        $this->response = $this->response->withBody($stream);
    }

    /**
     * import json style file
     */
    public function import() {
        try {
            if (empty($_FILES)) {
                throw new \Exception(__('No file to import'));
            }

            if ($_FILES['design']['type'] !== 'application/json') {
                throw new InvalidTypeException(__('Uploaded file type is not allowed'));
            }
            $fullPath = '/tmp/openITCOCKPIT-design.json';

            if (!move_uploaded_file($_FILES['design']['tmp_name'], $fullPath)) {
                throw new \Exception(__('Cannot move uploaded file'));
            }

            if (file_exists($fullPath)) {
                if ($content = file_get_contents($fullPath)) {
                    $content = json_decode($content, true);
                } else {
                    throw new \Exception('cannot read json file');
                }
            } else {
                throw new \Exception('file "' . $fullPath . '" does not exist');
            }

            /** @var DesignsTable $DesignsTable */
            $DesignsTable = TableRegistry::getTableLocator()->get('DesignModule.Designs');
            $design = $DesignsTable->getDesign();

            if (empty($design)) {
                $design = $DesignsTable->newEntity($content);
            } else {
                $design = $DesignsTable->patchEntity($design, $content);
            }

            $DesignsTable->save($design);
            if ($design->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $design->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            try {
                $content = require_once(PLUGIN . 'DesignModule' . DS . 'templates' . DS . 'Designs' . DS . 'css' . DS . 'style.php');
                $cssFile = PLUGIN . 'DesignModule' . DS . 'webroot' . DS . 'css' . DS . 'customStyle.css';
                $DesignsTable->writeStyleFile($cssFile, $content);
            } catch (\Exception $e) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', 'Cannot write style file: ' . $e->getMessage());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            $this->set('success', __('Design imported and saved successfully'));
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;

        } catch (\Exception $e) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $e->getMessage());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }
    }

    /**
     * reset to default style
     */
    public function reset() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            $themeType = $this->request->getQuery('type');
            /** @var DesignsTable $DesignsTable */
            $DesignsTable = TableRegistry::getTableLocator()->get('DesignModule.Designs');
            switch ($themeType) {
                case '1':
                    //dark theme
                    $theme = [
                        'pageHeader'          => '#403f3f',
                        'pageSidebar'         => '#434343',
                        'pageSidebarGradient' => '#323232',
                        'navTitle'            => '#afafaf',
                        'navMenu'             => '#a0a0a0',
                        'navMenuHover'        => '#f8f8f8',
                        'navLink'             => '#727b84',
                        'navLinkHover'        => '#333',
                        'navTab'              => '#363636',
                        'pageContent'         => '#bababa',
                        'pageContentWrapper'  => '#545454',
                        'panelHdr'            => '#232323',
                        'panel'               => '#383838',
                        'breadcrumbNonLinks'  => '#bfbfbf',
                        'breadcrumbLinks'     => '#fefefe',
                        'tableBase'           => '#3e3e3e',
                        'tableBaseFont'       => '#ababab',
                        'tableBorder'         => '#363636',
                        'tableHover'          => '#727272',
                        'tableHoverFont'      => '#ababab',
                        'btnDefault'          => '#2b2b2b',
                        'btnDefaultFont'      => '#dedede',
                        'btnDefaultBorder'    => '#787878',
                        'cardBase'            => '#535353',
                        'cardBaseFont'        => '#bababa',
                        'cardHeader'          => '#7a7a7a',
                        'cardHeaderFont'      => '#bababa',
                        'logoInHeader'        => 1,
                        'customcsstext'       => $DesignsTable->getCustomCss()
                    ];
                    break;

                default:
                    $theme = $this->defaultTheme;
                    $cssFile = PLUGIN . 'DesignModule' . DS . 'webroot' . DS . 'css' . DS . 'customStyle.css';
                    if (file_exists($cssFile)) {
                        unlink($cssFile);
                    }
                    $DesignsTable->deleteAll([]);
                    break;
            }
            $this->set('design', $theme);
            $this->viewBuilder()->setOption('serialize', ['design']);
        }
    }
}
