<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace DesignModule\Controller;

use Cake\Http\Exception\ForbiddenException;
use itnovum\openITCOCKPIT\Core\Views\Logo;

/**
 * Uploads Controller
 *
 *
 * @method \DesignModule\Model\Entity\Upload[]|\Cake\Datasource\ResultSetInterface paginate($object = null, array $settings = [])
 */
class UploadsController extends AppController {

    /**
     * @throws \Exception
     */
    public function uploadLogo(): void {
        if (empty($_FILES)) {
            throw new \Exception(__('There is no file to store'));
        }

        $paramName = key($_FILES);

        if ($_FILES[$paramName]['error'] !== UPLOAD_ERR_OK) {
            throw new \Exception(__('Error on uploading file'));
        }

        $logoPath = null;
        $logo = new Logo();

        switch ($paramName) {
            case 0: // interface and pdfs
                $logoPath = $logo->getCustomLogoPdfPath();
                break;
            case 1: // notifications
                $logoPath = $logo->getCustomSmallLogoPdfPath();
                break;
            case 2: // interface (big)
                $logoPath = $logo->getCustomLogoDiskPath();
                break;
            case 3: // header
                $logoPath = $logo->getCustomHeaderLogoDiskPath();
                break;
        }

        if ($logoPath == null) {
            throw new \Exception('Logo Type not known');
        }

        $tmpLogoFile = '/tmp/uploaded_logo.unknown.picture';

        if (!move_uploaded_file($_FILES[$paramName]['tmp_name'], $tmpLogoFile)) {
            throw new \Exception(__('Cannot move uploaded file'));
        }

        $imgsize = getimagesize($tmpLogoFile);
        $width = $imgsize[0];
        $height = $imgsize[1];
        $imgtype = $imgsize[2];

        switch ($imgtype) {
            /**
             * 1 => GIF
             * 2 => JPG
             * 3 => PNG
             * 4 => SWF
             * 5 => PSD
             * 6 => BMP
             * 7 => TIFF(intel byte order)
             * 8 => TIFF(motorola byte order)
             * 9 => JPC
             * 10 => JP2
             * 11 => JPX
             * 12 => JB2
             * 13 => SWC
             * 14 => IFF
             * 15 => WBMP
             * 16 => XBM
             */
            case 1:
                $srcImg = imagecreatefromgif($tmpLogoFile);
                break;
            case 2:
                $srcImg = imagecreatefromjpeg($tmpLogoFile);
                break;
            case 3:
                $srcImg = imagecreatefrompng($tmpLogoFile);
                break;
            default:
                //Filetype not supported!
                throw new \Exception(__('Unsuported image format.'));
                break;
        }

        imagesavealpha($srcImg, true);
        imagepng($srcImg, $logoPath);
        unlink($tmpLogoFile);

        $successMsg = __('Upload successful');
        $this->set('success', $successMsg);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    public function resetLogo() {
        try {
            $successMsg = __('Reset successful');
            $logo = new Logo();
            $logoPath = null;
            $paramName = $this->request->getQuery('type', 0);

            switch ($paramName) {
                case 0: //interface and pdfs
                    $logoPath = $logo->getCustomLogoPdfPath();
                    break;
                case 1: // notifications
                    $logoPath = $logo->getCustomSmallLogoPdfPath();
                    break;
                case 2: //interface big
                    $logoPath = $logo->getCustomLogoDiskPath();
                    break;
                case 3: //header logo
                    $logoPath = $logo->getCustomHeaderLogoDiskPath();
                    break;
            }

            if ($logoPath == null) {
                throw new \Exception('Logo Type not known');
            }

            if (!file_exists($logoPath)) {
                $successMsg = __('No custom logo found');
            } else if (!unlink($logoPath)) {
                throw new \Exception('Cannot delete custom logo');
            }

            $this->set('success', $successMsg);
            $this->viewBuilder()->setOption('serialize', ['success']);

        } catch (\Exception $e) {
            throw new ForbiddenException('Cannot delete custom logo file ' . $logoPath . ': ' . $e->getMessage());
        }
    }
}
