<?php
declare(strict_types=1);

namespace DesignModule\Model\Table;

use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * Designs Model
 *
 * @method \DesignModule\Model\Entity\Design newEmptyEntity()
 * @method \DesignModule\Model\Entity\Design newEntity(array $data, array $options = [])
 * @method \DesignModule\Model\Entity\Design[] newEntities(array $data, array $options = [])
 * @method \DesignModule\Model\Entity\Design get($primaryKey, $options = [])
 * @method \DesignModule\Model\Entity\Design findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \DesignModule\Model\Entity\Design patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \DesignModule\Model\Entity\Design[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \DesignModule\Model\Entity\Design|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \DesignModule\Model\Entity\Design saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \DesignModule\Model\Entity\Design[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \DesignModule\Model\Entity\Design[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \DesignModule\Model\Entity\Design[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \DesignModule\Model\Entity\Design[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class DesignsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('designs');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('pageHeader')
            ->maxLength('pageHeader', 255)
            ->requirePresence('pageHeader', 'create')
            ->notEmptyString('pageHeader');

        $validator
            ->scalar('pageSidebar')
            ->maxLength('pageSidebar', 255)
            ->requirePresence('pageSidebar', 'create')
            ->notEmptyString('pageSidebar');

        $validator
            ->scalar('pageSidebarGradient')
            ->maxLength('pageSidebarGradient', 255)
            ->requirePresence('pageSidebarGradient', 'create')
            ->notEmptyString('pageSidebarGradient');

        $validator
            ->scalar('navTitle')
            ->maxLength('navTitle', 255)
            ->requirePresence('navTitle', 'create')
            ->notEmptyString('navTitle');

        $validator
            ->scalar('navMenu')
            ->maxLength('navMenu', 255)
            ->requirePresence('navMenu', 'create')
            ->notEmptyString('navMenu');

        $validator
            ->scalar('navMenuHover')
            ->maxLength('navMenuHover', 255)
            ->requirePresence('navMenuHover', 'create')
            ->notEmptyString('navMenuHover');

        $validator
            ->scalar('navLink')
            ->maxLength('navLink', 255)
            ->requirePresence('navLink', 'create')
            ->notEmptyString('navLink');

        $validator
            ->scalar('navLinkHover')
            ->maxLength('navLinkHover', 255)
            ->requirePresence('navLinkHover', 'create')
            ->notEmptyString('navLinkHover');

        $validator
            ->scalar('navTab')
            ->maxLength('navTab', 255)
            ->requirePresence('navTab', 'create')
            ->notEmptyString('navTab');

        $validator
            ->scalar('pageContent')
            ->maxLength('pageContent', 255)
            ->requirePresence('pageContent', 'create')
            ->notEmptyString('pageContent');

        $validator
            ->scalar('pageContentWrapper')
            ->maxLength('pageContentWrapper', 255)
            ->requirePresence('pageContentWrapper', 'create')
            ->notEmptyString('pageContentWrapper');

        $validator
            ->scalar('panelHdr')
            ->maxLength('panelHdr', 255)
            ->requirePresence('panelHdr', 'create')
            ->notEmptyString('panelHdr');

        $validator
            ->scalar('panel')
            ->maxLength('panel', 255)
            ->requirePresence('panel', 'create')
            ->notEmptyString('panel');

        $validator
            ->scalar('breadcrumbNonLinks')
            ->maxLength('breadcrumbNonLinks', 255)
            ->requirePresence('breadcrumbNonLinks', 'create')
            ->notEmptyString('breadcrumbNonLinks');

        $validator
            ->scalar('breadcrumbLinks')
            ->maxLength('breadcrumbLinks', 255)
            ->requirePresence('breadcrumbLinks', 'create')
            ->notEmptyString('breadcrumbLinks');

        $validator
            ->scalar('tableBase')
            ->maxLength('tableBase', 255)
            ->requirePresence('tableBase', 'create')
            ->notEmptyString('tableBase');

        $validator
            ->scalar('tableBaseFont')
            ->maxLength('tableBaseFont', 255)
            ->requirePresence('tableBaseFont', 'create')
            ->notEmptyString('tableBaseFont');

        $validator
            ->scalar('tableBorder')
            ->maxLength('tableBorder', 255)
            ->requirePresence('tableBorder', 'create')
            ->notEmptyString('tableBorder');

        $validator
            ->scalar('tableHover')
            ->maxLength('tableHover', 255)
            ->requirePresence('tableHover', 'create')
            ->notEmptyString('tableHover');

        $validator
            ->scalar('tableHoverFont')
            ->maxLength('tableHoverFont', 255)
            ->requirePresence('tableHoverFont', 'create')
            ->notEmptyString('tableHoverFont');

        $validator
            ->scalar('btnDefault')
            ->maxLength('btnDefault', 255)
            ->requirePresence('btnDefault', 'create')
            ->notEmptyString('btnDefault');

        $validator
            ->scalar('btnDefaultFont')
            ->maxLength('btnDefaultFont', 255)
            ->requirePresence('btnDefaultFont', 'create')
            ->notEmptyString('btnDefaultFont');

        $validator
            ->scalar('btnDefaultBorder')
            ->maxLength('btnDefaultBorder', 255)
            ->requirePresence('btnDefaultBorder', 'create')
            ->notEmptyString('btnDefaultBorder');

        $validator
            ->scalar('cardBase')
            ->maxLength('cardBase', 255)
            ->requirePresence('cardBase', 'create')
            ->notEmptyString('cardBase');

        $validator
            ->scalar('cardBaseFont')
            ->maxLength('cardBaseFont', 255)
            ->requirePresence('cardBaseFont', 'create')
            ->notEmptyString('cardBaseFont');

        $validator
            ->scalar('cardHeader')
            ->maxLength('cardHeader', 255)
            ->requirePresence('cardHeader', 'create')
            ->notEmptyString('cardHeader');

        $validator
            ->scalar('cardHeaderFont')
            ->maxLength('cardHeaderFont', 255)
            ->requirePresence('cardHeaderFont', 'create')
            ->notEmptyString('cardHeaderFont');

        $validator
            ->requirePresence('logoInHeader', 'create')
            ->notEmptyString('logoInHeader');

        return $validator;
    }

    /**
     * @param int $id
     * @return bool
     */
    public function existsById($id) {
        return $this->exists(['Designs.id' => $id]);
    }


    public function getDesign() {
        return $this->find()->first();
    }

    /**
     * @param $path
     * @param $content
     * @return bool
     * @throws \Exception
     */
    public function writeStyleFile($path, $content): bool {
        try {
            $handle = fopen($path, 'w+');
            if (!$handle) {
                throw new \ErrorException('Cannot open style file handle');
            }

            if (!fwrite($handle, $content)) {
                throw new \ErrorException('Cannot write style file or content length is 0');
            }
            fclose($handle);
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @return string
     */
    public function getCustomCss() {
        $currentDesign = $this->find()->first();
        if (!empty($currentDesign)) {
            return $currentDesign['customcsstext'];
        }
        return '';
    }
}
